@@ -1,12 +1,12 @@
-;;; eshell-prompt-extras.el --- Display extra information for your eshell prompt.
+;;; eshell-prompt-extras.el --- Display extra information for your eshell prompt. -*- lexical-binding: t -*-
 
 ;; Copyright (C) 2014-2019 Wei Zhao
 
 ;; Author: zwild <judezhao@outlook.com>
 ;; Contributors: Lee Hinman
-;; Maintainer: Chunyang Xu <mail@xuchunyang.me>
+;; Maintainer: Xu Chunyang <xuchunyang56@gmail.com>
 ;; URL: https://github.com/zwild/eshell-prompt-extras
-;; Version: 1.0
+;; Version: 1.1
 ;; Created: 2014-08-16
 ;; Keywords: eshell, prompt
 ;; Package-Requires: ((emacs "25"))
@@ -88,41 +88,51 @@
 (autoload 'vc-find-root "vc-hooks")
 
 (defgroup epe nil
-  "Eshell extras"
+  "Display extra information for your eshell prompt."
   :group 'eshell-prompt)
 
 (defcustom epe-show-python-info t
-  "non nil will show python info."
+  "Whether epe-pipeline should show the python virtual environment info."
   :group 'epe
   :type 'boolean)
 
 (defcustom epe-git-dirty-char "*"
-  "Character to show for a changed git repository"
+  "The character to show when the git repository is dirty."
   :group 'epe
   :type 'string)
 
 (defcustom epe-git-untracked-char "?"
-  "Character to show for an untracked file in the git repository"
+  "The character to show when the git repository has untracked files."
   :group 'epe
   :type 'string)
 
 (defcustom epe-git-detached-HEAD-char "D:"
-  "Character to show for an detached HEAD in the git repository"
+  "The character to show when the git repository is in detached HEAD state."
   :group 'epe
   :type 'string)
 
+(defcustom epe-show-local-working-directory nil
+  "Whether epe-pipeline should show the local path of the working directory."
+  :group 'epe
+  :type 'boolean)
+
 (defcustom epe-path-style 'fish
   "Prompt path name style."
   :group 'epe
   :type '(choice (const :tag "fish-style-dir-name" fish)
-                 (const :tag "single-dir-name" single)
-                 (const :tag "full-path-name" full)))
+          (const :tag "single-dir-name" single)
+          (const :tag "full-path-name" full)))
 
 (defcustom epe-fish-path-max-len 30
   "Default maximum length for path in `epe-fish-path'."
   :group 'epe
   :type 'number)
 
+(defcustom epe-pipeline-show-time t
+  "A flag which indicates whether epe-pipeline should show the time."
+  :group 'epe
+  :type 'boolean)
+
 (defface epe-remote-face
   '((t (:inherit font-lock-comment-face)))
   "Face of remote info in prompt."
@@ -158,10 +168,22 @@
   :group 'epe)
 
 (defface epe-sudo-symbol-face
-  '((t (:inherit eshell-ls-unreadable-face)))
+  `((t (:inherit ,(if (facep 'eshell-ls-unreadable)
+                      'eshell-ls-unreadable
+                    'eshell-ls-unreadable-face))))
   "Face of your sudo symbol in prompt."
   :group 'epe)
 
+(defface epe-success-face
+  '((t (:inherit success)))
+  "Face of success info in prompt."
+  :group 'epe)
+
+(defface epe-error-face
+  '((t (:inherit error)))
+  "Face of failure info in prompt."
+  :group 'epe)
+
 (defface epe-pipeline-delimiter-face
   '((t :foreground "green"))
   "Face for pipeline theme delimiter."
@@ -190,23 +212,32 @@
 ;; help definations
 ;; (epe-colorize-with-face "abc" 'font-lock-comment-face)
 (defmacro epe-colorize-with-face (str face)
+  "Colorize STR with FACE."
   `(propertize ,str 'face ,face))
 
+(defun epe-pwd ()
+  "Return the current working directory-name."
+  (if epe-show-local-working-directory
+      (tramp-file-local-name (eshell/pwd))
+    (eshell/pwd)))
+
 (defun epe-abbrev-dir-name (dir)
-  "Return the base directory name."
+  "Return the base directory name of DIR."
   (if (string= dir (getenv "HOME"))
       "~"
     (let ((dirname (file-name-nondirectory dir)))
       (if (string= dirname "") "/" dirname))))
 
 (defun epe-trim-newline (string)
+  "Trim newline from the end of STRING."
   (replace-regexp-in-string "\n$" "" string))
 
 ;; https://www.emacswiki.org/emacs/EshellPrompt
 (defun epe-fish-path (path &optional max-len)
-  "Return a potentially trimmed-down version of the directory PATH, replacing
-parent directories with their initial characters to try to get the character
-length of PATH (sans directory slashes) down to MAX-LEN."
+  "Return a potentially trimmed-down version of the directory PATH.
+Replacing parent directories with their initial characters to try
+to get the character length of PATH (sans directory slashes) down
+to MAX-LEN."
   (let* ((components (split-string (abbreviate-file-name path) "/"))
          (max-len (or max-len epe-fish-path-max-len))
          (len (+ (1- (length components))
@@ -259,7 +290,7 @@ length of PATH (sans directory slashes) down to MAX-LEN."
     (getenv "USER")))
 
 (defun epe-date-time (&optional format)
-  "Date time information."
+  "Date time information in FORMAT."
   (format-time-string (or format "%Y-%m-%d %H:%M") (current-time)))
 
 (defun epe-status-formatter (timestamp duration)
@@ -279,6 +310,7 @@ time the command took to complete in seconds."
 (make-variable-buffer-local 'epe-status--last-command-time)
 
 (defun epe-status--record ()
+  "Record the time of the current command."
   (setq epe-status--last-command-time (current-time)))
 
 (defun epe-status (&optional formatter min-duration)
@@ -304,6 +336,7 @@ returns a string."
 
 ;; tramp info
 (defun epe-remote-p ()
+  "If you are in a remote machine."
   (tramp-tramp-file-p default-directory))
 
 (defun epe-remote-user ()
@@ -333,6 +366,7 @@ returns a string."
          (vc-find-root (eshell/pwd) ".git"))))
 
 (defun epe-git-short-sha1 ()
+  "Return the short sha1 of your git commit."
   (epe-trim-newline (shell-command-to-string "git rev-parse --short HEAD")))
 
 ;; (defun epe-git-branch ()
@@ -347,7 +381,7 @@ returns a string."
   (let ((branch (car (vc-git-branches))))
     (cond
      ((null branch) "no-branch")
-     ((string-match "^(HEAD detached at \\([[:word:]]+\\)" branch)
+     ((string-match "^(HEAD detached at \\(.+\\))$" branch)
       (concat epe-git-detached-HEAD-char (match-string 1 branch)))
      (t branch))))
 
@@ -375,7 +409,7 @@ uncommitted changes, nil otherwise."
           (match-string 1 it))))))
 
 (defun epe-git-dirty ()
-  "Return if your git is 'dirty'."
+  "Return if your git is \"dirty\"."
   (if (string-match "dirty"
                     (shell-command-to-string "git diff-index --quiet HEAD -- || echo -n 'dirty'"))
       epe-git-dirty-char ""))
@@ -386,46 +420,60 @@ uncommitted changes, nil otherwise."
    (shell-command-to-string "git log @{u}.. --oneline 2> /dev/null | wc -l")))
 
 (defun epe-git-untracked ()
+  "Return `epe-git-untracked-char' if your git has untracked files."
   (and (epe-git-untracked-p) epe-git-untracked-char))
 
 (defvar epe-git-status
   "git status --porcelain -b 2> /dev/null")
 
 (defun epe-git-p-helper (command)
+  "Return if COMMAND has output."
   (not (string= (shell-command-to-string command) "")))
 
 (defun epe-git-untracked-p ()
+  "Return if your git has untracked files."
   (epe-git-p-helper (concat epe-git-status " | grep '^\?\? '")))
 
 (defun epe-git-added-p ()
+  "Return if your git has added files."
   (or (epe-git-p-helper (concat epe-git-status " | grep '^A '"))
       (epe-git-p-helper (concat epe-git-status " | grep '^M '"))))
 
 (defun epe-git-modified-p ()
+  "Return if your git has modified files."
   (or (epe-git-p-helper (concat epe-git-status " | grep '^ M '"))
       (epe-git-p-helper (concat epe-git-status " | grep '^AM '"))
       (epe-git-p-helper (concat epe-git-status " | grep '^ T '"))))
 
 (defun epe-git-renamed-p ()
+  "Return if your git has renamed files."
   (epe-git-p-helper (concat epe-git-status " | grep '^R '")))
 
 (defun epe-git-deleted-p ()
+  "Return if your git has deleted files."
   (or (epe-git-p-helper (concat epe-git-status " | grep '^ D '"))
       (epe-git-p-helper (concat epe-git-status " | grep '^D '"))
       (epe-git-p-helper (concat epe-git-status " | grep '^AD '"))))
 
 (defun epe-git-unmerged-p ()
+  "Return if your git has unmerged files."
   (epe-git-p-helper (concat epe-git-status " | grep '^UU '")))
 
 (defun epe-git-ahead-p ()
+  "Return if your git has ahead commits."
   (epe-git-p-helper (concat epe-git-status " | grep '^## .*ahead'")))
 
 (defun epe-git-behind-p ()
+  "Return if your git has behind commits."
   (epe-git-p-helper (concat epe-git-status " | grep '^## .*behind'")))
 
 (defun epe-git-diverged-p ()
+  "Return if your git has diverged commits."
   (epe-git-p-helper (concat epe-git-status " | grep '^## .*deverged'")))
 
+(defvar eshell-prompt-regexp)
+(defvar venv-current-name)
+(defvar conda-env-current-name)
 
 ;;; Themes
 ;; Please post your theme here if you want.
@@ -438,12 +486,14 @@ uncommitted changes, nil otherwise."
      (epe-colorize-with-face
       (concat (epe-remote-user) "@" (epe-remote-host) " ")
       'epe-remote-face))
-   (when (and epe-show-python-info (bound-and-true-p venv-current-name))
-     (epe-colorize-with-face (concat "(" venv-current-name ") ") 'epe-venv-face))
+   (let ((env-current-name (or (bound-and-true-p venv-current-name)
+                               (bound-and-true-p conda-env-current-name))))
+     (when (and epe-show-python-info (bound-and-true-p env-current-name))
+       (epe-colorize-with-face (concat "(" env-current-name ") ") 'epe-venv-face)))
    (let ((f (cond ((eq epe-path-style 'fish) 'epe-fish-path)
                   ((eq epe-path-style 'single) 'epe-abbrev-dir-name)
                   ((eq epe-path-style 'full) 'abbreviate-file-name))))
-     (epe-colorize-with-face (funcall f (eshell/pwd)) 'epe-dir-face))
+     (epe-colorize-with-face (funcall f (epe-pwd)) 'epe-dir-face))
    (when (epe-git-p)
      (concat
       (epe-colorize-with-face ":" 'epe-dir-face)
@@ -455,7 +505,9 @@ uncommitted changes, nil otherwise."
                  (unless (= unpushed 0)
                    (concat ":" (number-to-string unpushed)))))
        'epe-git-face)))
-   (epe-colorize-with-face " λ" 'epe-symbol-face)
+   (epe-colorize-with-face " λ" (if (zerop eshell-last-command-status)
+                                    'epe-success-face
+                                  'epe-error-face))
    (epe-colorize-with-face (if (= (user-uid) 0) "#" "") 'epe-sudo-symbol-face)
    " "))
 
@@ -494,8 +546,8 @@ uncommitted changes, nil otherwise."
        (epe-colorize-with-face (concat "(" venv-current-name ") ") 'epe-venv-face))
      (epe-colorize-with-face (funcall
                               shrink-paths
-                              (split-string
-                               (funcall pwd-repl-home (eshell/pwd)) "/"))
+                              (split-string (funcall pwd-repl-home (epe-pwd))
+                                            "/"))
                              'epe-dir-face)
      (when (epe-git-p)
        (concat
@@ -507,7 +559,9 @@ uncommitted changes, nil otherwise."
                  (unless (= (epe-git-unpushed-number) 0)
                    (concat ":" (number-to-string (epe-git-unpushed-number)))))
          'epe-git-face)))
-     (epe-colorize-with-face " λ" 'epe-symbol-face)
+     (epe-colorize-with-face " λ" (if (zerop eshell-last-command-status)
+                                      'epe-success-face
+                                    'epe-error-face))
      (epe-colorize-with-face (if (= (user-uid) 0) "#" "") 'epe-sudo-symbol-face)
      " ")))
 
@@ -517,22 +571,25 @@ uncommitted changes, nil otherwise."
   (concat
    (if (epe-remote-p)
        (progn
-	     (concat
-	      (epe-colorize-with-face "┌─[" 'epe-pipeline-delimiter-face)
-	      (epe-colorize-with-face (epe-remote-user) 'epe-pipeline-user-face)
-	      (epe-colorize-with-face "@" 'epe-pipeline-delimiter-face)
-	      (epe-colorize-with-face (epe-remote-host) 'epe-pipeline-host-face)))
+	 (concat
+	  (epe-colorize-with-face "┌─[" 'epe-pipeline-delimiter-face)
+	  (epe-colorize-with-face (epe-remote-user) 'epe-pipeline-user-face)
+	  (epe-colorize-with-face "@" 'epe-pipeline-delimiter-face)
+	  (epe-colorize-with-face (epe-remote-host) 'epe-pipeline-host-face)))
      (progn
        (concat
-	    (epe-colorize-with-face "┌─[" 'epe-pipeline-delimiter-face)
-	    (epe-colorize-with-face (user-login-name) 'epe-pipeline-user-face)
-	    (epe-colorize-with-face "@" 'epe-pipeline-delimiter-face)
-	    (epe-colorize-with-face (system-name) 'epe-pipeline-host-face))))
+	(epe-colorize-with-face "┌─[" 'epe-pipeline-delimiter-face)
+	(epe-colorize-with-face (user-login-name) 'epe-pipeline-user-face)
+	(epe-colorize-with-face "@" 'epe-pipeline-delimiter-face)
+	(epe-colorize-with-face (system-name) 'epe-pipeline-host-face))))
    (concat
     (epe-colorize-with-face "]──[" 'epe-pipeline-delimiter-face)
-    (epe-colorize-with-face (format-time-string "%H:%M" (current-time)) 'epe-pipeline-time-face)
-    (epe-colorize-with-face "]──[" 'epe-pipeline-delimiter-face)
-    (epe-colorize-with-face (concat (eshell/pwd)) 'epe-dir-face)
+    (when epe-pipeline-show-time
+      (concat
+       (epe-colorize-with-face
+        (format-time-string "%H:%M" (current-time)) 'epe-pipeline-time-face)
+       (epe-colorize-with-face "]──[" 'epe-pipeline-delimiter-face)))
+    (epe-colorize-with-face (epe-pwd) 'epe-dir-face)
     (epe-colorize-with-face  "]\n" 'epe-pipeline-delimiter-face)
     (epe-colorize-with-face "└─>" 'epe-pipeline-delimiter-face))
    (when (and epe-show-python-info (bound-and-true-p venv-current-name))
@@ -542,19 +599,19 @@ uncommitted changes, nil otherwise."
       (epe-colorize-with-face ":" 'epe-dir-face)
       (epe-colorize-with-face
        (concat (epe-git-branch)
-	           (epe-git-dirty)
-	           (epe-git-untracked)
-	           (let ((unpushed (epe-git-unpushed-number)))
-		         (unless (= unpushed 0)
-		           (concat ":" (number-to-string unpushed)))))
+	       (epe-git-dirty)
+	       (epe-git-untracked)
+	       (let ((unpushed (epe-git-unpushed-number)))
+		 (unless (= unpushed 0)
+		   (concat ":" (number-to-string unpushed)))))
        'epe-git-face)))
    (epe-colorize-with-face " λ" 'epe-symbol-face)
    (epe-colorize-with-face (if (= (user-uid) 0) "#" "") 'epe-sudo-symbol-face)
    " "))
 
 (defun epe-theme-multiline-with-status ()
-  "A simple eshell-prompt theme with information on its own line
-and status display on command termination."
+  "A simple eshell-prompt theme with information on its own line.
+The status is displayed on the last line."
   ;; If the prompt spans over multiple lines, the regexp should match
   ;; last line only.
   (setq eshell-prompt-regexp "^> ")
@@ -569,7 +626,7 @@ and status display on command termination."
    (let ((f (cond ((eq epe-path-style 'fish) 'epe-fish-path)
                   ((eq epe-path-style 'single) 'epe-abbrev-dir-name)
                   ((eq epe-path-style 'full) 'abbreviate-file-name))))
-     (pcase (epe-extract-git-component (funcall f (eshell/pwd)))
+     (pcase (epe-extract-git-component (funcall f (epe-pwd)))
        (`(,prefix nil)
         (format
          (propertize "[%s]" 'face '(:weight bold))
